/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CAN Loopback Sample Application. \n
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2020
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "bsp.h"
#include "bsp_uart_io.h"
#include "bsp_timer.h"
#include "adc.h"
#include "can.h"
#include "sys_timer.h"

/**
 *  @addtogroup Example Example
 *  @{
 */

/** 
 *  @addtogroup CAN_Loopback CAN_Loopback
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CAN_Loopback_config CAN_Loopback Config
 *  @{
 */

/** 
 *  @addtogroup CountConfiguration Time Configuration
 *  @brief    Interval Time(Interrupt Cycle).
 *  @{
 */
#define CFG_OUTPUT_COUNT_TIME           ((uint32_t)(5000))
/** 
 *  @}
 */ /* End of group CountConfiguration */

/** 
 *  @}
 */ /* End of group CAN_Loopback_config */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CAN_Loopback_Private_define CAN_Loopback Private Define
 *  @{
 */
#define MAIN_NULL       ((void *)0)     /*!< Null Pointer  */
#define USER_ID_UART    ((uint32_t)1)   /*!< User ID :USB UART (Any value) */
#define CFG_ADC_COUNT_TIME              ((uint32_t)(1000))

#define MSG_ID                      (0x00000014U)
#define MSG_BOX_TX_BITNO            (0x00000001U)
#define MSG_BOX_RX_BITNO            (0x00000400U)

#define MSG_BOX_NO_0                (0x00000001U)
#define MSG_BOX_NO_10               (0x00000400U)

/**
 *  @}
 */ /* End of group CAN_Loopback_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ADC_Monitor_Private_typedef ADC_Monitor Private Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    t32a_t      t32a;              /*!< Driver      :T32A.     */
    adc_t       adc;               /*!< Driver      :ADC.      */
    uart_io_t   *p_uart;           /*!< Application :USB UART. */
    timer_t     timer;             /*!< Application :Timer (1ms timer). */
    can_t       can;               /*!< Driver      :CAN.      */
} instance_t;

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CAN_Loopback_Private_variables CAN_Loopback Private Variables
 *  @{
 */
static instance_t instance;                             /*!< Instance.                */
static volatile uint32_t output_flag = 0;               /*!< Output flag.   */
uint32_t data0 = 0;                                     /*!< ADC value AIN0     */
uint32_t data1 = 0;                                     /*!< ADC value AIN1(AIN0) */
volatile uint32_t rev0 = 0;                             /*!< MB10 DL value */
volatile uint32_t rev1 = 0;                             /*!< MB10 DH value */
static uint32_t counter  = CFG_OUTPUT_COUNT_TIME;       /*!< ADC value output Time Counter  */
static uint32_t adc_counter  = CFG_ADC_COUNT_TIME;      /*!< AD-Convert Time Counter  */
static volatile uint8_t fIntADC = 0U;
/**
 *  @}
 */ /* End of group CAN_Loopback_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CAN_Loopback_Private_fuctions CAN_Loopback Private Fuctions
 *  @{
 */
static void variable_initialize(void);
static TXZ_Result driver_initialize(void);
static TXZ_Result application_initialize(void);
static void timer_interval_handler(uint32_t id);

/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.adc.p_instance     = MAIN_NULL;
    instance.p_uart             = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Uart                 */
    /*----------------------*/
    /* Driver initialize will process within USB UART. */
    /*----------------------*/
    /* ADC                  */
    /*----------------------*/
    {
        adc_t *p_adc = &instance.adc;

        /*--- Construct  ---*/
        /* Register Allocation */
        p_adc->p_instance       = TSB_ADC;
        /*--- Initialize ---*/
        p_adc->init.id          = (uint32_t)p_adc;
        p_adc->init.clk.exaz0    = ADC_SAMPLING_PERIOD0_X2N;
        p_adc->init.clk.exaz1    = ADC_SAMPLING_PERIOD1_XN;
        p_adc->init.clk.vadcld  = ADC_SCLK_4;   /* less than 40MHz  160MHz / 4 = 40MHz */
        p_adc->init.mod1        = ADC_MOD1_AVDD5_5V_SCLK_40;
        p_adc->init.mod2        = ADC_MOD2_TMPM4Mx;
        
        if (adc_init(p_adc) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* CAN                  */
    /*----------------------*/
    {
        can_t *p_can = &instance.can;
        uint32_t gsr_status;
        uint32_t tmp = 0;
        p_can->p_instance = TSB_CANA;
        can_set_software_reset(p_can);
        wait(10);
        p_can->bcr1 = 0x00000001;
        can_set_bcr1(p_can);
        p_can->bcr2.sjw = CANBCR2_SJW_4;
        p_can->bcr2.sam = CANBCR2_SAM_SINGLE;
        p_can->bcr2.tseg2 = CANBCR2_TSEG2_8;
        p_can->bcr2.tseg1 = CANBCR2_TSEG1_11;
        can_set_bcr2(p_can);
        p_can->mcr.sur = CANMCR_SUR_SUSPENDCLR;
        p_can->mcr.tstlb = CANMCR_TSTLB_LBREQ;
        p_can->mcr.tsterr = CANMCR_TSTERR_ERRCLR;
        p_can->mcr.ccr = CANMCR_CCR_CCRCLR;
        p_can->mcr.smr = CANMCR_SMR_SMRCLR;
        p_can->mcr.wuba = CANMCR_WUBA_WRITE;
        p_can->mcr.mtos = CANMCR_MTOS_LITTLE;
        can_set_mcr(p_can);
        can_get_gsr(p_can, &gsr_status);
        while((gsr_status & (0x0000001 << 7)) != 0) {
            can_get_gsr(p_can, &gsr_status);
        }

        /* TX config */
        p_can->p_instance2 = TSB_CANAMB0;
        /* Disable mailbox: set [CANMC]<MCn> to '0'*/
        can_get_mc(p_can, &tmp);
        p_can->mc = tmp & ~MSG_BOX_TX_BITNO;
        can_set_mc(p_can);
        /* Configure mailbox No.0 for reception: set [CANMC]<MDn> to '0'*/
        can_get_md(p_can, &tmp);
        p_can->md = tmp & ~MSG_BOX_TX_BITNO;
        can_set_md(p_can);
        /* Setup mailbox No.0 identifier and [CANMBnID]<IDE> bit for Standard ID or extended ID set <RFH> to '1' */
        p_can->id.ide = CANMBID_IDE_STANDARD;
        p_can->id.gamelame = CANMBID_GAMELAME_DISABLE;
        p_can->id.rfh = CANMBID_RFH_RES;
        p_can->id.id = MSG_ID;
        can_set_id(p_can);
        /* Setup mailbox No.0 set [CANMBnTSVMCF]<RTR> to '0' and <DLC[3:0]> to '1000' */
        p_can->tsvmcf.tsv = 0;
        p_can->tsvmcf.rtr = CANMBTSVMCF_RTR_DATA;
        p_can->tsvmcf.dlc = CANMBTSVMCF_DLC_8;
        can_set_tsvmcf(p_can);
        /* Enable mailbox No.0 : set [CANMC]<MCn> to '1' */
        can_get_mc(p_can, &tmp);
        p_can->mc = tmp | MSG_BOX_TX_BITNO;
        can_set_mc(p_can);

        /* RX config */
        p_can->p_instance2 = TSB_CANAMB10;
        /* Configure mailbox No.10 for reception: set [CANMC]<MDn> to '1'*/
        can_get_md(p_can, &tmp);
        p_can->md = tmp | MSG_BOX_RX_BITNO;
        can_set_md(p_can);
        /* Setup mailbox No.10 identifier and [CANMBnID]<IDE> bit for Standard ID or extended ID set <GAME_LAME> to '1', <RFH> to '1' */
        p_can->id.ide = CANMBID_IDE_STANDARD;
        p_can->id.gamelame = CANMBID_GAMELAME_ENABLE;
        p_can->id.rfh = CANMBID_RFH_RES;
        p_can->id.id = MSG_ID;
        can_set_id(p_can);
        /* Setup mailbox No.10 set [CANMBnTSVMCF]<RTR> to '0' and <DLC[3:0]> to '1000' */
        p_can->tsvmcf.tsv = 0;
        p_can->tsvmcf.rtr = CANMBTSVMCF_RTR_DATA;
        p_can->tsvmcf.dlc = CANMBTSVMCF_DLC_8;
        can_set_tsvmcf(p_can);
        /* Enable mailbox No.10 : set [CANMC]<MCn> to '1' */
        can_get_mc(p_can, &tmp);
        p_can->mc = tmp | MSG_BOX_RX_BITNO;
        can_set_mc(p_can);
        /* Reset data mailbox10 */
        p_can->transdata.data = 0x0U;
        p_can->transdata.datafield = CANMB_DATAL;
        can_set_transdata(p_can);
        p_can->transdata.datafield = CANMB_DATAH;
        can_set_transdata(p_can);
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @retval TXZ_SUCCESS :Success
  * @retval TXZ_ERROR   :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }

    /*----------------------*/
    /* Usb Uart             */
    /*----------------------*/
    /* Tx & Rx use same UART channel. */
    {
        uart_io_initial_setting_t param;

        param.id         = USER_ID_UART;
        switch (bsp_get_usb_uart_tx_ch(BSP_USB_UART_0))
        {
        case 0:
            param.p_reg  = TSB_UART0;
            break;
        case 1:
            param.p_reg  = TSB_UART1;
            break;
        case 2:
            param.p_reg  = TSB_UART2;
            break;
        default:
            param.p_reg  = MAIN_NULL;
            break;
        }
        param.boudrate   = bsp_get_usb_uart_tx_baudrate(BSP_USB_UART_0);
        param.irq.tx     = bsp_get_usb_uart_tx_nvic(BSP_USB_UART_0);
        param.irq.rx     = bsp_get_usb_uart_rx_nvic(BSP_USB_UART_0);
        param.irq.err    = bsp_get_usb_uart_err_nvic(BSP_USB_UART_0);
        if (param.p_reg != MAIN_NULL)
        {
            result  = uart_io_initialize(&param);
        }
        else
        {
            result = TXZ_ERROR;
        }
    }

    /*----------------------*/
    /* ADC                  */
    /*----------------------*/
    {
        adc_t *p_adc = &instance.adc;
        adc_channel_setting_t param;
        param.interrupt = ADC_INT_ENABLE;
        param.type = ADC_CONVERSION_SGL;
        param.ain = bsp_get_adc_adin(BSP_ADC_0);
        if (adc_channel_setting(p_adc, bsp_get_adc_ch(BSP_ADC_0), &param) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }
    
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id        :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        /* Increment Tick. Tick is used to driver's Timeout. */
        hal_inc_tick();
        /* Interval Time Count. */
        if(adc_counter-- == 0)
        {
            fIntADC = 1;
            adc_counter = CFG_ADC_COUNT_TIME;
            adc_start(&instance.adc);
        }
        if(counter-- == 0)
        {
            output_flag = 1;
            counter = CFG_OUTPUT_COUNT_TIME;
        }
    }
}

/**
 *  @}
 */ /* End of group CAN_Loopback_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CAN_Loopback_Exported_functions CAN_Loopback Exported Functions
 *  @{
 */
 
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    uint32_t reg;

    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- variable    ---*/
    variable_initialize();
    /*--- driver      ---*/
    if (driver_initialize() != TXZ_SUCCESS)
    {
    }
    /*--- application ---*/
    if (application_initialize() != TXZ_SUCCESS)
    {
    }
    /* Timer Start */
    timer_start(&instance.timer);

    /*------------------------*/
    /*          Loop          */
    /*------------------------*/ 
    while(1)
    {
        if( fIntADC == 1 )
        {
            fIntADC = 0;
            adc_channel_get_value(&instance.adc, bsp_get_adc_ch(BSP_ADC_0), &data0);
            data1 = data0;

            /* Copy data to CANMBxDH/CANMBxDL */
            /* Put the ADC result into mailbox0 */
            instance.can.p_instance2 = TSB_CANAMB0;
            instance.can.transdata.data = data0;
            instance.can.transdata.datafield = CANMB_DATAL;
            can_set_transdata(&instance.can);

            instance.can.transdata.data = data1;
            instance.can.transdata.datafield = CANMB_DATAH;
            can_set_transdata(&instance.can);

            can_get_trs(&instance.can, &reg);
            if((reg & MSG_BOX_NO_0) == 0)
            {
                /* mailbox No.0 Start to send */
                instance.can.trs = MSG_BOX_NO_0;
                can_set_trs(&instance.can);
            }
            
            /* Polling TX, Wait TX Ack */
            can_get_ta(&instance.can, &reg);
            while((reg & MSG_BOX_NO_0) == 0)
            {
                can_get_ta(&instance.can, &reg);
            }

            /* Pending Check */
            can_get_rmp(&instance.can, &reg);
            while((reg & MSG_BOX_NO_10) == 0)
            {
                can_get_rmp(&instance.can, &reg);
            }

            /* Read Receive data */
            instance.can.p_instance2 = TSB_CANAMB10;
            instance.can.receivedata.datafield = CANMB_DATAL;
            can_get_receivedata(&instance.can);
            rev0 = instance.can.receivedata.data;

            instance.can.receivedata.datafield = CANMB_DATAH;
            can_get_receivedata(&instance.can);
            rev1 = instance.can.receivedata.data;

            /* Clear Pending */
            instance.can.rmp |= MSG_BOX_NO_10;
            can_set_rmp(&instance.can);
        }
            
        /* ADC Result Display */
        if( output_flag == 1 )
        {
            printf("nAIN0 > 0x%04x\n", rev0);
            printf("nAIN1 > 0x%04x\n\n", rev1);

            /* output flag clear */
            output_flag = 0;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_rx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_tx_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    switch (uart)
    {
    /*--- usb uart 0 ---*/
    case BSP_USB_UART_0:
        uart_io_err_irq_handler();
        break;
    /*--- default    ---*/
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group CAN_Loopback_Exported_functions */

/**
 *  @}
 */ /* End of group CAN_Loopback */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
