/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CG_EXINT Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include "bsp.h"
#include "bsp_sw.h"
#include "bsp_led.h"
#include "bsp_timer.h"
#include "cg.h"
#include "flash.h"
#include "port.h"
#include "t32a.h"
#include "siwdt.h"
#include "uart.h"
#include "uart_ex.h"
#include "sys_timer.h"
/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup CG_EXINT CG_EXINT Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_EXINT_Private_macro CG_EXINT Private Macro
 *  @{
 */



/** 
 *  @}
 */ /* End of group CG_EXINT_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_EXINT_Private_define CG_EXINT Private Define
 *  @{
 */
/** 
 *  @defgroup LEDBlinkConfiguration LED Blink Time Configuration
 *  @brief    LED Blink Interval Time(ms).
 *  @{
 */
#define CFG_LED_BLINK_CYCLE             ((uint32_t)(1000))
#define CFG_LED_BLINK_FRQ               ((uint32_t)(2))
#define CFG_LED_BLINK_DUTY              ((float)(0.5))
#define CFG_LED_ON_TIME   ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ * CFG_LED_BLINK_DUTY))
#define CFG_LED_OFF_TIME  ((uint32_t)(CFG_LED_BLINK_CYCLE / CFG_LED_BLINK_FRQ * (1 - CFG_LED_BLINK_DUTY)))
/** 
 *  @}
 */ /* End of group LEDBlinkConfiguration */

/** 
 *  @defgroup SWPollingConfiguration SW Polling Time Configuration
 *  @brief    SW Polling Time(ms).
 *  @{
 */
#define CFG_SW_POLLING_TIME     ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group SWPollingConfiguration */

/** 
 *  @defgroup IHOSCConfiguration IHOSC Configuration
 *  @brief    IHOSC Configuration.
 *  @{
 */
#define CFG_IHOSC_WARM_UP_TIME  ((uint32_t)(5000))      /*!< Warm up time(us).  Minimam warm-up time is 100(us). */
#define CFG_IHOSC_CLOCK         ((uint32_t)(10000000))  /*!< Clock(hz).        */
/** 
 *  @}
 */ /* End of group IHOSCConfiguration */

/** 
 *  @}
 */ /* End of group CG_EXINT_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_EXINT_Private_define CG_EXINT Private Define
 *  @{
 */

#define MAIN_NULL               ((void *)0) /*!< Null Pointer */

/** 
 *  @}
 */ /* End of group CG_EXINT_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_EXINT_Private_define CG_EXINT Private Define
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,              /*!< Idle.                      */
    PROC_NORMAL_START,      /*!< Normal Mode    :START.     */
    PROC_NORMAL,            /*!< Normal Mode    :RUNNING.   */
    PROC_NORMAL_END,        /*!< Normal Mode    :END.       */
    PROC_LOW_POWER_START,   /*!< Low Power Mode :START.     */
    PROC_LOW_POWER_END,     /*!< Low Power Mode :END.       */
    PROC_ERROR              /*!< Error.                     */
} Processing;

/*----------------------------------*/
/** 
 * @brief  Low Power Mode.
*/
/*----------------------------------*/
typedef enum
{
    LOW_POWER_IDLE,         /*!< Idle Mode.     */
    LOW_POWER_STOP_1,       /*!< Stop1 Mode.    */
    LOW_POWER_STOP_2,       /*!< Stop2 Mode.    */
} LowPowerMode;
/** 
 *  @}
 */ /* End of group CG_EXINT_Private_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_EXINT_Private_typedef CG_EXINT Private Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
   t32a_t      t32a;                       /*!< Driver      :T32A               */
   timer_t     timer;                      /*!< Application :Timer (1ms timer). */
   sw_t        sw[(uint32_t)BSP_PSW_MAX];  /*!< Application :Push SW            */
   led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
   rlm_t       rlm;                        /*!< Driver      :RLM.      */
   cg_t        cg;                         /*!< Driver      :CG.       */
   flash_t     flash;                      /*!< Driver      :FLASH.       */
} instance_t;

/**
 *  @}
 */ /* End of group CG_EXINT_Private_typedef */


/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_EXINT_Private_variables CG_EXINT Private Variables
 *  @{
 */
static instance_t   instance;  /*!< Instance.                    */
static volatile Processing   proc;      /*!< Processing mode.             */
static volatile LowPowerMode low;       /*!< Select Low Power Mode.       */
static rlm_t        *p_rlm ;   /*!< RLM */
static cg_t         *p_cg ;    /*!< CG */
/**
 *  @}
 */ /* End of group CG_EXINT_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_EXINT_Private_fuctions CG_EXINT Private Fuctions
 *  @{
 */

static void variable_initialize(void);
static void timer_interval_handler(uint32_t id);
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state);
static TXZ_Result driver_initialize_normal(void);
static TXZ_Result driver_finalize_normal(void);
static TXZ_Result application_initialize_normal(void);
static TXZ_Result application_finalize_normal(void);
static TXZ_Result normal_start(void);
static TXZ_Result normal_end(void);
static TXZ_Result driver_initialize_idle(void);
static TXZ_Result driver_finalize_idle(void);
static TXZ_Result application_initialize_idle(void);
static TXZ_Result application_finalize_idle(void);
static TXZ_Result low_power_start_idle(void);
static TXZ_Result low_power_end_idle(void);
static TXZ_Result driver_initialize_stop1(void);
static TXZ_Result driver_finalize_stop1(void);
static TXZ_Result application_initialize_stop1(void);
static TXZ_Result application_finalize_stop1(void);
static TXZ_Result low_power_start_stop1(void);
static TXZ_Result low_power_end_stop1(void);
#if (STOP2_EQUIPPED == AVAILABLE)
static TXZ_Result driver_initialize_stop2(void);
static TXZ_Result application_initialize_stop2(void);
static TXZ_Result low_power_start_stop2(void);
#endif









/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance    = MAIN_NULL;
    instance.cg.p_instance      = MAIN_NULL;
    instance.rlm.p_instance     = MAIN_NULL;
    instance.timer.init.p_t32a  = MAIN_NULL;
    instance.flash.p_instance   = MAIN_NULL;
    {
        uint32_t i;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            instance.sw[i].init.p_gpio  = MAIN_NULL;
        }
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            instance.led[i].init.p_gpio = MAIN_NULL;
        }
    }
    /* Processing mode. */
    proc = PROC_NORMAL_START;
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        uint32_t i;

        /*----------------------*/
        /* Tick                 */
        /*----------------------*/
        /* Increment Tick. Tick is used to driver's Timeout. */
        hal_inc_tick();
        /*----------------------*/
        /* SW                   */
        /*----------------------*/
        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            sw_1ms_timer_handler(&instance.sw[i]);
        }
        /*----------------------*/
        /* LED                  */
        /*----------------------*/
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            led_1ms_timer_handler(&instance.led[i]);
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  SW State Change Handler.
  * @param  id    :User ID
  * @param  state :Push SW Port State
  * @return -
  * @retval -
  * @note   Notify that, "State is changed." \n
  *         Call by SW Task \n
  *         If SW is pushed, change the processing mode(@ref Processing).
  */
/*--------------------------------------------------*/
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state)
{
    /* Check SW Push(= GPIO_PIN_RESET) */
    if (state == GPIO_PIN_RESET)
    {
        uint32_t i;

        for (i=0; (i < (uint32_t)BSP_PSW_MAX) && (proc != PROC_NORMAL_END); i++)
        {
            if (id == (uint32_t)&instance.sw[i])
            {
                /* Select Low Power Mode */
                switch ((BSPPsw)i)
                {
                case BSP_PSW_1:
                    low  = LOW_POWER_IDLE;
                    proc = PROC_NORMAL_END;
                    break;
                case BSP_PSW_2:
                    low = LOW_POWER_STOP_1;
                    proc = PROC_NORMAL_END;
                    break;
#if (STOP2_EQUIPPED == AVAILABLE)
                case BSP_PSW_3:
                    low = LOW_POWER_STOP_2;
                    proc = PROC_NORMAL_END;
                    break;
#endif
                default:
                    /* no processing */
                    break;
                }
            }
        }
    }
}

/*==========================*/
/* Normal Mode              */
/*==========================*/
/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for Normal Mode
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize_normal(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */
    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
        {
            result = TXZ_ERROR;
        }
    }

    {
        p_rlm = &instance.rlm;
        p_cg  = &instance.cg;

        p_rlm->p_instance = TSB_RLM;
        p_cg->p_instance = TSB_CG;
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for Normal Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize_normal(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Finalize ---*/
        /* no processing */
        /*--- Destruct ---*/
        /* Register Release */
        p_t32a->p_instance = MAIN_NULL;
    }
    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for Normal Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize_normal(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id          = (uint32_t)p_led;
            p_led->init.p_gpio      = bsp_get_gpio_instance_address();
            p_led->init.port.group  = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num    = bsp_get_gpio_num_led((BSPLed)(i));
            /* LED Normal :blinking */
            if (i == (uint32_t)BSP_LED_1)
            {
                p_led->init.state               = LED_STATE_ON;
                p_led->init.blink.func          = TXZ_ENABLE;
                p_led->init.blink.interval.on   = CFG_LED_ON_TIME;
                p_led->init.blink.interval.off  = CFG_LED_OFF_TIME;
            }
            /* default    :off      */
            else
            {
                p_led->init.state               = LED_STATE_OFF;
                p_led->init.blink.func          = TXZ_DISABLE;
                p_led->init.blink.interval.on   = 0;
                p_led->init.blink.interval.off  = 0;
            }
            led_initialize(p_led);
        }
    }
    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    {
        uint32_t i;
        sw_t *p_sw = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            p_sw = &instance.sw[i];

            p_sw->init.id                   = (uint32_t)p_sw;
            p_sw->init.p_gpio               = bsp_get_gpio_instance_address();
            p_sw->init.port.group           = bsp_get_gpio_group_psw((BSPPsw)(i));
            p_sw->init.port.num             = bsp_get_gpio_num_psw((BSPPsw)(i));
            p_sw->init.polling.func         = TXZ_ENABLE;
            p_sw->init.polling.interval     = CFG_SW_POLLING_TIME;
            p_sw->init.chattering.func      = TXZ_DISABLE;
            p_sw->init.chattering.count     = 0;
            p_sw->init.handler              = sw_state_change_handler;
            sw_initialize(p_sw);
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for Normal Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize_normal(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
    {
        sw_finalize(&instance.sw[i]);
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
    {
        led_finalize(&instance.led[i]);
    }
    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    timer_finalize(&instance.timer);

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Start Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for Normal Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result normal_start(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* Driver               */
    /*----------------------*/
    if (driver_initialize_normal() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*----------------------*/
    /* Application          */
    /*----------------------*/
    if (application_initialize_normal() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*----------------------*/
    /* 1ms Timer Start      */
    /*----------------------*/
    timer_start(&instance.timer);

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  End Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for Normal Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result normal_end(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* 1ms Timer Stop       */
    /*----------------------*/
    timer_stop(&instance.timer);
    /*----------------------*/
    /* Application          */
    /*----------------------*/
    if (application_finalize_normal() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*----------------------*/
    /* Driver               */
    /*----------------------*/
    if (driver_finalize_normal() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*==========================*/
/* IDLE Mode                */
/*==========================*/
/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for IDLE Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize_idle(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for IDLE Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize_idle(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for IDLE Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize_idle(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            /* LED IDLE :on  */
            if (i == (uint32_t)BSP_LED_2)
            {
                p_led->init.state           = LED_STATE_ON;
            }
            /* default  :off */
            else
            {
                p_led->init.state           = LED_STATE_OFF;
            }
            led_initialize(p_led);
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for IDLE Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize_idle(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
    {
        led_finalize(&instance.led[i]);
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Start Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for IDLE Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result low_power_start_idle(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    flash_t *p_flash = &instance.flash;

    p_flash->p_instance = TSB_FC;
    p_flash->init.status = FC_SR0_RDYBSY;

    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    wdt_disable(TSB_SIWD0);
    /*----------------------*/
    /* Flash Wait           */
    /*----------------------*/
    /* [FCSR0]<RDYBSY>   : Read(wait for done) */
    while(fc_get_status(p_flash) == TXZ_BUSY){
    };
    /*----------------------*/
    /* Set STBY             */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>    :IDLE            */
    cg_set_stby_mode(p_cg, CG_STBY_IDLE);
    /*----------------------*/
    /* STBY Wait            */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>   : Read(wait for done) */
    while (cg_get_stby_mode(p_cg) != CG_STBY_IDLE)
    {
        /* no processing */
    }
    /*----------------------*/
    /* Driver               */
    /*----------------------*/
    if (driver_initialize_idle() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*----------------------*/
    /* Application          */
    /*----------------------*/
    if (application_initialize_idle() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*---------------------------*/
    /* External Interrupt Enable */
    /*---------------------------*/
    /* Exint Enable        */
    bsp_irq_exint_enable();
    /* NVIC Enable */
    NVIC_EnableIRQ(bsp_get_exint_nvic());

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  End Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for IDLE Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result low_power_end_idle(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------------*/
    /* External Interrupt Disable */
    /*----------------------------*/
    /* NVIC Disable         */
    NVIC_DisableIRQ(bsp_get_exint_nvic());
    /* IRQ Disable          */
    bsp_irq_exint_disable();
    /*----------------------*/
    /* Application          */
    /*----------------------*/
    if (application_finalize_idle() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*----------------------*/
    /* Driver               */
    /*----------------------*/
    if (driver_finalize_idle() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

/*==========================*/
/* STOP1 Mode               */
/*==========================*/
/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for STOP1 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize_stop1(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Driver Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for STOP1 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result driver_finalize_stop1(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver finalize will process within BSP.     */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for STOP1 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize_stop1(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            /* LED STOP1 :on  */
            if (i == (uint32_t)BSP_LED_3)
            {
                p_led->init.state           = LED_STATE_ON;
            }
            /* default   :off */
            else
            {
                p_led->init.state           = LED_STATE_OFF;
            }
            led_initialize(p_led);
        }
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Finalize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for STOP1 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result application_finalize_stop1(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    uint32_t i;

    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
    {
        led_finalize(&instance.led[i]);
    }

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Low Power Mode Start Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   STOP1 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result low_power_start_stop1(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    flash_t *p_flash = &instance.flash;

    p_flash->p_instance = TSB_FC;
    p_flash->init.status = FC_SR0_RDYBSY;

    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    wdt_disable(TSB_SIWD0);
    /*----------------------*/
    /* Flash Wait           */
    /*----------------------*/
    /* [FCSR0]<RDYBSY>   : Read(wait for done) */
    while(fc_get_status(p_flash) == TXZ_BUSY){
    };
    /*----------------------*/
    /* HOSC Warm-up Wait    */
    /*----------------------*/
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    cg_wait_warmup();
    /*----------------------*/
    /* Internal HOSC Warm-up Time Set*/
    /*----------------------*/
    cg_set_warm_up_time(p_cg, CG_WARM_UP_SRC_IHOSC1, WU_TIME_INT);
    /*----------------------*/
    /* Set STBY             */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>    :STOP1           */
    cg_set_stby_mode(p_cg, CG_STBY_STOP1);
    /*----------------------*/
    /* PLL Disable          */
    /*----------------------*/
    /* [CGPLL0SEL]<PLL0SEL>: fosc           */
    cg_set_PLL_clock(CG_PLL0SEL_FOSC);
    /* [CGPLL0SEL]<PLL0ST> : wait for fosc  */
    while (cg_get_PLL_status() != CG_PLL0SEL_FOSC) 
    {
        /* no processing */
    }
    /* [CGPLL0SEL]<PLL0ON> : PLL Disable    */
    cg_PLL_disable();
    /*----------------------*/
    /* Internal HOSC Enable */
    /*----------------------*/
    /* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
    cg_ihosc1_enable();
    /* [CGWUPHCR]<WUON> : Warm up Start    */
    cg_start_warmup();
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    cg_wait_warmup();
    /* [CGOSCCR]<OSCSEL>   : IHOSC          */
    cg_set_oscsel(CG_OSCSEL_IHOSC1);
    /* [CGOSCCR]<OSCF>     : Read(wait for IHOSC) */
    while (cg_get_oscsel() != CG_OSCSEL_IHOSC1) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* External HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : EHOSC Disable  */
    cg_set_ehoscen(p_cg, CG_EHOSCEN_UNUSE);
    /*----------------------*/
    /* OFD Internal HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2EN> : OFD IHOSC Disable  */
    cg_ihosc2_disable();
    /*----------------------*/
    /* External HOSC Disable Check*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : Read  */
    while (cg_get_ehoscen(p_cg) != CG_EHOSCEN_UNUSE) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* OFD Internal HOSC Disable Wait*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2F>  : Read(wait for OFD IHOSC Disable)  */
    while (cg_get_ihosc2_state() == TXZ_DONE) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* Driver               */
    /*----------------------*/
    if (driver_initialize_stop1() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*----------------------*/
    /* Application          */
    /*----------------------*/
    if (application_initialize_stop1() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*---------------------------*/
    /* External Interrupt Enable */
    /*---------------------------*/
    /* Exint Enable        */
    bsp_irq_exint_enable();
    /* NVIC Enable */
    NVIC_EnableIRQ(bsp_get_exint_nvic());

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Low Power Mode End Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   STOP1 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result low_power_end_stop1(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    
    /*----------------------------*/
    /* External Interrupt Disable */
    /*----------------------------*/
    /* NVIC Disable         */
    NVIC_DisableIRQ(bsp_get_exint_nvic());
    /* IRQ Disable          */
    bsp_irq_exint_disable();
    /*----------------------*/
    /* Application          */
    /*----------------------*/
    if (application_finalize_stop1() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*----------------------*/
    /* Driver               */
    /*----------------------*/
    if (driver_finalize_stop1() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }

    return (result);
}

#if (STOP2_EQUIPPED == AVAILABLE)
/*==========================*/
/* STOP2 Mode               */
/*==========================*/
/*--------------------------------------------------*/
/** 
  * @brief  Driver Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for STOP2 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result driver_initialize_stop2(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* GPIO                 */
    /*----------------------*/
    /* Driver initialize has done within BSP. */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for STOP2 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize_stop2(void)
{
    TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    /* LED initialize has done within BSP. */

    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  Start Processing.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   for STOP2 Mode.
  */
/*--------------------------------------------------*/
static TXZ_Result low_power_start_stop2(void)
{
    TXZ_Result result = TXZ_SUCCESS;
    flash_t *p_flash = &instance.flash;

    p_flash->p_instance = TSB_FC;
    p_flash->init.status = FC_SR0_RDYBSY;

    /*----------------------*/
    /* Application          */
    /*----------------------*/
    /*--- LED ---*/
    /* Set LED Port before IO Keep. */
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            /* LED STOP2 :on  */
            if (i == (uint32_t)BSP_LED_4)
            {
                p_led->init.state           = LED_STATE_ON;
            }
            /* default   :off */
            else
            {
                p_led->init.state           = LED_STATE_OFF;
            }
            led_initialize(p_led);
        }
    }
    /*----------------------*/
    /* WDT Disable          */
    /*----------------------*/
    wdt_disable(TSB_SIWD0);
    /*----------------------*/
    /* Flash Wait           */
    /*----------------------*/
    /* [FCSR0]<RDYBSY>   : Read(wait for done) */
    while(fc_get_status(p_flash) == TXZ_BUSY){
    };
    /*----------------------*/
    /* IO Keep              */
    /*----------------------*/
    /* RLMSHTDNOP:<PTKEEP=1>   */
    rlm_port_keep_enable(p_rlm) ;
    /*----------------------*/
    /* Set STBY             */
    /*----------------------*/
    /* [CGSTBYCR]<STBY>    :STOP2           */
    cg_set_stby_mode(p_cg, CG_STBY_STOP2);
    /*----------------------*/
    /* PLL Disable          */
    /*----------------------*/
    /* [CGPLL0SEL]<PLL0SEL>: fosc           */
    cg_set_PLL_clock(CG_PLL0SEL_FOSC);
    /* [CGPLL0SEL]<PLL0ST> : wait for fosc  */
    while (cg_get_PLL_status() != CG_PLL0SEL_FOSC) 
    {
        /* no processing */
    }
    /* [CGPLL0SEL]<PLL0ON> : PLL Disable    */
    cg_PLL_disable();
    /*----------------------*/
    /* Internal HOSC Warm-up Time Set*/
    /*----------------------*/
    cg_set_warm_up_time(p_cg, CG_WARM_UP_SRC_IHOSC1, WU_TIME_INT);
    /*----------------------*/
    /* Internal HOSC Enable */
    /*----------------------*/
    /* [CGOSCCR]<IHOSC1EN> : HOSC Enable    */
    cg_ihosc1_enable();
    /* [CGWUPHCR]<WUON> : Warm up Start    */
    cg_start_warmup();
    /* [CGWUPHCR]<WUEF>  : Read(wait for done) */
    cg_wait_warmup();
    /* [CGOSCCR]<OSCSEL>   : IHOSC          */
    cg_set_oscsel(CG_OSCSEL_IHOSC1);
    /* [CGOSCCR]<OSCF>     : Read(wait for IHOSC) */
    while (cg_get_oscsel() != CG_OSCSEL_IHOSC1) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* External HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : EHOSC Disable  */
    cg_set_ehoscen(p_cg, CG_EHOSCEN_UNUSE);
    /*----------------------*/
    /* OFD Internal HOSC Disable*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2EN> : OFD IHOSC Disable  */
    cg_ihosc2_disable();
    /*----------------------*/
    /* External HOSC Disable Check*/
    /*----------------------*/
    /* [CGOSCCR]<EOSCEN>   : Read  */
    while (cg_get_ehoscen(p_cg) != CG_EHOSCEN_UNUSE) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* OFD Internal HOSC Disable Wait*/
    /*----------------------*/
    /* [CGOSCCR]<IHOSC2F>  : Read(wait for OFD IHOSC Disable)  */
    while (cg_get_ihosc2_state() == TXZ_DONE) 
    {
        /* no processing */
    }
    /*----------------------*/
    /* STOP2 Reset Flag RESET Pin Flag Clear*/
    /*----------------------*/
    /* [RLMRSTFLG0]<STOP2RSTF>,<PINRSTF>  : Clear */
    rlm_clear_flag0(p_rlm, RLM_PINRSTF);
    rlm_clear_flag0(p_rlm, RLM_STOP2RSTF);
    /*----------------------*/
    /* Driver               */
    /*----------------------*/
    if (driver_initialize_stop2() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /*----------------------*/
    /* Application          */
    /*----------------------*/
    if (application_initialize_stop2() != TXZ_SUCCESS)
    {
        result = TXZ_ERROR;
    }
    /* Exint Enable        */
    bsp_irq_exint_enable();
    /* NVIC Enable */
    NVIC_EnableIRQ(bsp_get_exint_nvic());

    return (result);
}
#endif
/**
 *  @}
 */ /* End of group CG_EXINT_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CG_EXINT_Exported_functions CG_EXINT Exported Functions
 *  @{
 */
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
#if (STOP2_EQUIPPED == AVAILABLE)
    {
        p_rlm = &instance.rlm;
        p_cg  = &instance.cg;

        p_rlm->p_instance = TSB_RLM;
        p_cg->p_instance = TSB_CG;
    }
    /*----------------------*/
    /* Reset Flag Check     */
    /*----------------------*/
    if (rlm_get_flag0(p_rlm, RLM_STOP2RSTF) == INTERRUPT_DETECT)
    {
        /* It's  executed, when STOP2 mode is released. */
        /*----------------------*/
        /* BSP initialize       */
        /*----------------------*/
        bsp_initialize();
        /*----------------------*/
        /* Release "IO Keep"    */
        /*----------------------*/
        rlm_port_keep_disable(p_rlm);
        /* Clear Flag */
        rlm_clear_flag0(p_rlm, RLM_PORSTF);
        rlm_clear_flag0(p_rlm, RLM_PINRSTF);
        rlm_clear_flag0(p_rlm, RLM_STOP2RSTF);
        rlm_clear_flag0(p_rlm, RLM_LVDRSTF);
        rlm_clear_flag1(p_rlm, RLM_SYSRSTF);
        rlm_clear_flag1(p_rlm, RLM_LOCKRSTF);
        rlm_clear_flag1(p_rlm, RLM_WDTRSTF);
        rlm_clear_flag1(p_rlm, RLM_OFDRSTF);
    }else
    {
        /*----------------------*/
        /* BSP initialize       */
        /*----------------------*/
        bsp_initialize();
    }

    /*--- Jump to the Interrupt routine, STOP2 mode is released. ---*/
    /*  STOP2 REQ Interrupt Enable */
    bsp_irq_exint_enable();
    /* NVIC Enable */
    NVIC_EnableIRQ(bsp_get_exint_nvic());
#else
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
#endif
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    variable_initialize();
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(proc != PROC_ERROR)
    {
        switch (proc)
        {
        /*--- Normal Mode     :START   ---*/
        case PROC_NORMAL_START:
            {
                if (normal_start() == TXZ_SUCCESS)
                {
                    proc = PROC_NORMAL;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Normal Mode    :RUNNING ---*/
        case PROC_NORMAL:
            {
                uint32_t i;
                /*----------------------*/
                /* SW                   */
                /*----------------------*/
                for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
                {
                    sw_task(&instance.sw[i]);
                }
                /*----------------------*/
                /* LED                  */
                /*----------------------*/
                for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
                {
                    led_task(&instance.led[i]);
                }
            }
            break;
        /*--- Normal Mode    :END     ---*/
        case PROC_NORMAL_END:
            {
                /*----------------------*/
                /* End Processing       */
                /*----------------------*/
                if (normal_end() == TXZ_SUCCESS)
                {
                    /*----------------------*/
                    /* BSP finalize         */
                    /*----------------------*/
                    /* if need, BSP finalize for normal mode. */
                    bsp_finalize();
                    proc = PROC_LOW_POWER_START;
                }
                else
                {
                    proc = PROC_ERROR;
                }
            }
            break;
        /*--- Low Power Mode :START   ---*/
        case PROC_LOW_POWER_START:
            {
                switch (low)
                {
                case LOW_POWER_IDLE:
                    {
                        /*----------------------*/
                        /* BSP initialize       */
                        /*----------------------*/
                        /* If need, BSP initialize for IDLE Mode. (Now, use Norma Mode) */
                        bsp_initialize();
                        /*----------------------*/
                        /* Start Processing     */
                        /*----------------------*/
                        if (low_power_start_idle() == TXZ_SUCCESS)
                        {
                            /*>>>Wait For Interrupt */
                            __WFI();
                            /*<<<Wait For Interrupt */
                            proc = PROC_LOW_POWER_END;
                        }
                        else
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    break;
                case LOW_POWER_STOP_1:
                    {
                        /*----------------------*/
                        /* BSP initialize       */
                        /*----------------------*/
                        /* If need, BSP initialize for Stop1 Mode. (Now, use Norma Mode) */
                        bsp_initialize();
                        /*----------------------*/
                        /* Start Processing     */
                        /*----------------------*/
                        if (low_power_start_stop1() == TXZ_SUCCESS)
                        {
                            /*>>>Wait For Interrupt */
                            __WFI();
                            /*<<<Wait For Interrupt */
                            proc = PROC_LOW_POWER_END;
                        }
                        else
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    break;
#if (STOP2_EQUIPPED == AVAILABLE)
                case LOW_POWER_STOP_2:
                    {
                        /*----------------------*/
                        /* BSP initialize       */
                        /*----------------------*/
                        /* If need, BSP initialize for Stop2 Mode. (Now, use Norma Mode) */
                        bsp_initialize();
                        /*----------------------*/
                        /* Start Processing     */
                        /*----------------------*/
                        /* When release the "Low Power Mode", reset sequence. */
                        if (low_power_start_stop2() == TXZ_SUCCESS)
                        {
                            /*>>>Wait For Interrupt */
                            while(1)
                            {
                                __WFI();
                            }
                            /*<<<Wait For Interrupt */
                            /*--- Reset ---*/
                        }
                        else
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    break;
#endif
                default:
                    proc = PROC_ERROR;
                    break;
                }
            }
            break;
        /*--- Low Power Mode :END     ---*/
        case PROC_LOW_POWER_END:
            {
                switch (low)
                {
                case LOW_POWER_IDLE:
                    {
                        /*----------------------*/
                        /* End Processing       */
                        /*----------------------*/
                        if (low_power_end_idle() == TXZ_SUCCESS)
                        {
                            /*----------------------*/
                            /* BSP finalize         */
                            /*----------------------*/
                            /* If need, BSP finalize for IDLE Mode. (Now, use Norma Mode) */
                            bsp_finalize();
                            /*----------------------*/
                            /* BSP initialize       */
                            /*----------------------*/
                            bsp_initialize();
                            /*----------------------*/
                            /* Initialize           */
                            /*----------------------*/
                            variable_initialize();
                            /*<<< If need, Re-Setting for Normal Mode. */
                        }
                        else
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    break;
                case LOW_POWER_STOP_1:
                    {
                        /*----------------------*/
                        /* End Processing       */
                        /*----------------------*/
                        if (low_power_end_stop1() == TXZ_SUCCESS)
                        {
                            /*----------------------*/
                            /* BSP finalize         */
                            /*----------------------*/
                            /* If need, BSP finalize for IDLE Mode. (Now, use Norma Mode) */
                            bsp_finalize();

                            /*>>> If need, Re-Setting for Normal Mode. */
                            /*----------------------*/
                            /* System Initialize    */
                            /*----------------------*/
                            SystemInit();
                            /*----------------------*/
                            /* BSP initialize       */
                            /*----------------------*/
                            bsp_initialize();
                            /*----------------------*/
                            /* Initialize           */
                            /*----------------------*/
                            variable_initialize();
                            /*<<< If need, Re-Setting for Normal Mode. */
                        }
                        else
                        {
                            proc = PROC_ERROR;
                        }
                    }
                    break;
#if (STOP2_EQUIPPED == AVAILABLE)
                case LOW_POWER_STOP_2:
#endif
                default:
                    /* Stop2 :Reset Sequence. */
                    proc = PROC_ERROR;
                    break;
                }
            }
            break;
        /*--- Error                   ---*/
        case PROC_ERROR:
            /* no processing */
            break;
        /*--- default                 ---*/
        default:
            proc = PROC_ERROR;
            break;
        }
    }
    /*>>> System Error. Please Debug. */
    /*----------------------*/
    /* BSP finalize         */
    /*----------------------*/
    bsp_finalize();
#ifdef __DEBUG__
    assert_failed(MAIN_NULL, 0);
#endif

    return (0);
}
/*--------------------------------------------------*/
/** 
  * @brief  External Interrupt IRQ.
  * @param  exi : BSPExInt.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_exint(BSPExInt exi)
{
    /*----------------------------*/
    /* External Interrupt Disable */
    /*----------------------------*/
    /* NVIC Disable         */
    NVIC_DisableIRQ(bsp_get_exint_nvic());
    /* IRQ Disable          */
    bsp_irq_exint_disable();
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/**
 *  @}
 */ /* End of group CG_EXINT_Exported_functions */

/**
 *  @}
 */ /* End of group CG_EXINT */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
