/** 
 *******************************************************************************
 * @file    main.c
 * @brief   I2C SLAVE TRANS Sample Application.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2023
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "bsp.h"
#include "bsp_i2c.h"
#include "bsp_led.h"
#include "bsp_sw.h"
#include "bsp_timer.h"
#include "sys_timer.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup I2C_SlaveTrans I2C_SlaveTrans Sample Appli
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SlaveTrans_Private_macro I2C_SlaveTrans Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_SlaveTrans_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SlaveTrans_Private_define I2C_SlaveTrans Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group I2C_SlaveTrans_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SlaveTrans_Private_define I2C_SlaveTrans Private Define
 *  @{
 */

#define MAIN_NULL       ((void *)0)         /*!< Null Pointer.  */
#define SELF_ADDR       ((uint8_t)0x60 << 1U)     /*!< Self address for test.        */
#define SLAVE_ADDR      ((uint8_t)0x60 << 1U)     /*!< Slave address for test.       */
#define DATA_LENGTH     (16)                /*!< Length of base data.          */
#define TEST_TIMEOUT    (16000000)          /*!< for verification.             */
#define MCU_NAME        "TMPM4KNFYA"
 /** 
 *  @defgroup SWPollingConfiguration SW Polling Time Configuration
 *  @brief    SW Polling Time(ms).
 *  @{
 */
#define CFG_SW_POLLING_TIME     ((uint32_t)(100))
/** 
 *  @}
 */ /* End of group SWPollingConfiguration */
/** 
 *  @defgroup I2CFrequencyConfiguration I2C Frequency Configuration
 *  @brief    I2C Frequency(hz).
 *  @{
 */
#define CFG_I2C_FREQUENCY       ((uint32_t)400000)    /*!< Fm   400kHz               */
/** 
 *  @}
 */ /* End of group I2CFrequencyConfiguration */
/** 
 *  @}
 */ /* End of group I2C_SlaveTrans_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SlaveTrans_Private_define I2C_SlaveTrans Private Define
 *  @{
 */

/*----------------------------------*/
/** 
 * @enum   Processing
 * @brief  Processing mode.
*/
/*----------------------------------*/
typedef enum
{
    PROC_IDLE,                    /*!< Idle.                        */
    PROC_SLAVE_START,             /*!< Slave start.                 */
    PROC_SLAVE_WRITE,             /*!< Slave write.                 */
    PROC_SLAVE_WRITE_SUCCESS,     /*!< Slave write success.         */
    PROC_SLAVE_END,               /*!< Slave end.                   */
    PROC_ERROR                    /*!< Error.                       */
} Processing;

/** 
 *  @}
 */ /* End of group I2C_SlaveTrans_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
   t32a_t       t32a;                       /*!< Driver      :T32A               */
   timer_t      timer;                      /*!< Application :Timer (1ms timer). */
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    sw_t        sw[(uint32_t)BSP_PSW_MAX];  /*!< Application :Push SW            */
} instance_t;

/** 
 *  @}
 */ /* End of group I2C_SlaveTrans_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SlaveTrans_Private_variables I2C_SlaveTrans Private Variables
 *  @{
 */

static instance_t instance;                     /*!< Instance.                */
static volatile Processing proc;                         /*!< Processing mode.         */

static i2c_t i2c_obj;                           /*!< I2C driver instance.    */
static uint32_t self_address;                   /*!< I2C Self Address.       */
static volatile uint32_t transfer_event;        /*!< I2C Transfer Event      */
static int32_t frequency;                       /*!< Frequency.               */

static int32_t timeout;                         /*!< Timeout work.            */
static uint32_t ledflag;                        /*!< Led Flag.                */
static uint8_t  slave_tx_data[DATA_LENGTH];     /*!< Tx Data of Slave.         */
static uint8_t mcu_name[] = MCU_NAME;           /*!< MCU NAME data.           */
static uint8_t data_size  = (uint8_t)(sizeof( MCU_NAME ) - 1U); /*!< MCU NAME data size.      */
/**
 *  @}
 */ /* End of group I2C_SlaveTrans_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SlaveTrans_Private_functions I2C_SlaveTrans Private Functions
 *  @{
 */
static void variable_initialize(void);
static TXZ_Result application_initialize(void);
static void configuration(void);
static void clear_slave_tx_data(void);
static void ts_i2c_initialize(void);
static void ts_i2c_set_slave_mode(void);
static void timer_interval_handler(uint32_t id);
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state);
static void ts_i2c_slave_non_blocking_trans(void);
static void ts_i2c_slave_abort_check(void);

/*--------------------------------------------------*/
/** 
  * @brief  Variable Initialize
  * @param  -
  * @return -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void variable_initialize(void)
{
    /* Instance. */
    instance.t32a.p_instance          = MAIN_NULL;
    instance.timer.init.p_t32a        = MAIN_NULL;
    {
        uint32_t i;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            instance.sw[i].init.p_gpio  = MAIN_NULL;
        }
        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            instance.led[i].init.p_gpio = MAIN_NULL;
        }
    }
    proc = PROC_IDLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;

    /*----------------------*/
    /* T32A                 */
    /*----------------------*/
    {
        t32a_t *p_t32a = &instance.t32a;

        /*--- Construct  ---*/
        /* Register Allocation */
        switch (bsp_get_timer_ch(BSP_TIMER_1MS))
        {
        case 0:
            p_t32a->p_instance = TSB_T32A0;
            break;
        case 1:
            p_t32a->p_instance = TSB_T32A1;
            break;
        case 2:
            p_t32a->p_instance = TSB_T32A2;
            break;
        case 3:
            p_t32a->p_instance = TSB_T32A3;
            break;
        case 4:
            p_t32a->p_instance = TSB_T32A4;
            break;
        case 5:
            p_t32a->p_instance = TSB_T32A5;
            break;
        default:
            p_t32a->p_instance = MAIN_NULL;
            break;
        }
        /*--- Initialize ---*/
        p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
        p_t32a->init_mode.mode.mode = T32A_MODE_16;
        if (p_t32a->p_instance != MAIN_NULL)
        {
            if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
            {
                result = TXZ_ERROR;
            }
        }
        else
        {
            result = TXZ_ERROR;
        }
    }
    /*----------------------*/
    /* Timer                */
    /*----------------------*/
    /* 1ms timer */
    {
        timer_t *p_timer = &instance.timer;

        p_timer->init.id       = (uint32_t)&instance.timer;
        p_timer->init.p_t32a   = &instance.t32a;
        p_timer->init.type     = bsp_get_timer_type(BSP_TIMER_1MS);
        p_timer->init.irq      = bsp_get_timer_nvic(BSP_TIMER_1MS);
        p_timer->init.interval = 1000;
        p_timer->init.handler  = timer_interval_handler;
        timer_initialize(p_timer);
    }
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            p_led->init.state           = LED_STATE_OFF;
            led_initialize(p_led);
        }
    }
    /*----------------------*/
    /* PSW                  */
    /*----------------------*/
    {
        uint32_t i;
        sw_t *p_sw = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            p_sw = &instance.sw[i];

            p_sw->init.id                   = (uint32_t)p_sw;
            p_sw->init.p_gpio               = bsp_get_gpio_instance_address();
            p_sw->init.port.group           = bsp_get_gpio_group_psw((BSPPsw)(i));
            p_sw->init.port.num             = bsp_get_gpio_num_psw((BSPPsw)(i));
            p_sw->init.polling.func         = TXZ_ENABLE;
            p_sw->init.polling.interval     = CFG_SW_POLLING_TIME;
            p_sw->init.chattering.func      = TXZ_DISABLE;
            p_sw->init.chattering.count     = 0;
            p_sw->init.handler              = sw_state_change_handler;
            sw_initialize(p_sw);
        }
    }
    return (result);
}

/*--------------------------------------------------*/
/** 
  * @brief  SW State Change Handler.
  * @param  id    :User ID
  * @param  state :Push SW Port State
  * @return -
  * @retval -
  * @note   Notify that, "State is changed." \n
  *         Call by SW Task \n
  *         If SW is pushed, change the processing mode(@ref Processing).
  */
/*--------------------------------------------------*/
static void sw_state_change_handler(uint32_t id, gpio_pinstate_t state)
{
    /* Check SW Push(= GPIO_PIN_RESET) */
    if (state == GPIO_PIN_RESET)
    {
        uint32_t i;

        for (i=0; i < (uint32_t)BSP_PSW_MAX; i++)
        {
            if (id == (uint32_t)&instance.sw[i])
            {
                /* Select Low Power Mode */
                switch ((BSPPsw)i)
                {
                case BSP_PSW_1:
                    proc = PROC_SLAVE_START;
                    break;
                default:
                    /* no processing */
                    break;
                }
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Interval Time Handler.
  * @param  id :User ID
  * @return -
  * @retval -
  * @note   for 1ms Timer.
  * @note   Call by Timer.
  */
/*--------------------------------------------------*/
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&instance.timer)
    {
        uint32_t i;

        /*----------------------*/
        /* Tick                 */
        /*----------------------*/
        /* Increment Tick. Tick is used to driver's Timeout. */
        hal_inc_tick();
        /*----------------------*/
        /* SW                   */
        /*----------------------*/
        for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
        {
            sw_1ms_timer_handler(&instance.sw[i]);
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Configuration
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void configuration(void)
{
    self_address    = (uint32_t)SELF_ADDR;
    frequency       = CFG_I2C_FREQUENCY;
    ledflag = 0U;
}

/*--------------------------------------------------*/
/** 
  * @brief  clear_slave_tx_data
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void clear_slave_tx_data(void)
{
    uint8_t i;

    for (i = 0; i < DATA_LENGTH; i++)
    {
        if(i < data_size){
            slave_tx_data[i] = mcu_name[i];
        }
        else{
            slave_tx_data[i] = 0x55;
        }
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Initializing of I2C Driver
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_initialize(void)
{
    /* Initializing of I2C */
    if (i2c_init(&i2c_obj, I2C_CFG_PORT_SDA, I2C_CFG_PORT_SCL) == TXZ_SUCCESS)
    {
        if (frequency != 100000)
        {
            /* Update frequency */
            if (i2c_frequency(&i2c_obj, frequency) != TXZ_SUCCESS)
            {
                proc = PROC_ERROR;
            }
        }
    }
    else
    {
        /* fatal error */
        proc = PROC_ERROR;
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Changing to Slave Mode
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_set_slave_mode(void)
{
    i2c_slave_address(&i2c_obj, self_address);
    i2c_slave_mode(&i2c_obj, 1);
}

/*--------------------------------------------------*/
/**
  * @brief     Trans by Slave non-blocking
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_slave_non_blocking_trans(void)
{
    TXZ_Result result_mem ;
    uint8_t write_size ;

    write_size = data_size ;
    transfer_event = 0;

    result_mem = TXZ_ERROR ;
    result_mem = i2c_slave_transfer_asynch(&i2c_obj, 
                        &slave_tx_data[0], 
                        write_size, 
                        I2C_NULL, 
                        0
                        ) ;


    if ( result_mem != TXZ_SUCCESS )
    {
        proc = PROC_ERROR;
    }
    else
    {
        while (1)
        {
            volatile uint32_t event = transfer_event;   /* asynch information */

            if ((event & I2C_EVENT_ALL) != 0)
            {
                /* event occured */
                if (event & I2C_EVENT_TRANSFER_COMPLETE)
                {
                    /* complete */
                    i2c_abort_asynch(&i2c_obj);
                }
                else if (event & I2C_EVENT_TRANSFER_EARLY_NACK)
                {
                    /* early complete */
                    i2c_slave_abort_asynch(&i2c_obj);
                    proc = PROC_ERROR;
                }
                else  /* if (event & I2C_EVENT_ERROR) */
                {
                    /* other error */
                    i2c_slave_abort_asynch(&i2c_obj);
                    proc = PROC_ERROR;
                }
                break;
            }
            else
            {
                /* no event */
                /* wait forever */
            }
        }   
    }
}

/*--------------------------------------------------*/
/**
  * @brief     Checking abort of slave
  * @param     -
  * @retval    -
  */
/*--------------------------------------------------*/
static void ts_i2c_slave_abort_check(void)
{
    timeout = TEST_TIMEOUT;
    while (i2c_active(&i2c_obj))
    {
        /* checking bus release */
        if ((timeout--) == 0)
        {
            i2c_reset(&i2c_obj);
            i2c_slave_address(&i2c_obj, self_address);
            i2c_slave_mode(&i2c_obj, 1);
            proc = PROC_ERROR;
            break;
        }           
    }
}

/**
 *  @}
 */ /* End of group I2C_SlaveTrans_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup I2C_SlaveTrans_Exported_functions I2C_SlaveTrans Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/**
  * @brief      Implementation of INTI2Cx_IRQHandler
  * @param      -
  * @retval     -
  */
/*--------------------------------------------------*/
void ts_i2cst_irq_handler(void)
{
    uint32_t event = 0;

    event = i2c_slave_irq_handler_asynch(&i2c_obj);
    if (event != 0)
    {
        transfer_event = event;
    }
}

/*--------------------------------------------------*/
/**
  * @brief      Implementation of INTI2Cx_IRQHandler
  * @param      -
  * @retval     -
  */
/*--------------------------------------------------*/
void ts_i2ctberbf_irq_handler(void)
{
    uint32_t event = 0;

    event = i2c_slave_irq_handler_asynch(&i2c_obj);

    if (event != 0)
    {
        transfer_event = event;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /* Variable initialize */
    variable_initialize();
    /* Application initialization */
    application_initialize();  
    configuration();
    clear_slave_tx_data();
    ts_i2c_initialize();
    timer_start(&instance.timer);
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/
    while(1)
    {
        switch(proc)
        {
        case PROC_IDLE:
            {
                uint32_t i;
                /*----------------------*/
                /* SW                   */
                /*----------------------*/
                for (i=0; i<(uint32_t)BSP_PSW_MAX; i++)
                {
                    sw_task(&instance.sw[i]);
                }
            }
            break;

        case PROC_SLAVE_START:
            ts_i2c_set_slave_mode();
            proc = PROC_SLAVE_WRITE;
            led_turn_off(&instance.led[2]);
            break;

        case PROC_SLAVE_WRITE:
            ts_i2c_set_slave_mode();
            ts_i2c_slave_abort_check();
            clear_slave_tx_data();
            proc = PROC_SLAVE_WRITE_SUCCESS;
            ts_i2c_slave_non_blocking_trans();
            break;

        case PROC_SLAVE_WRITE_SUCCESS:
            if(ledflag == 1U){
                led_turn_off(&instance.led[1]);
                ledflag = 0U;
            }else{
                led_turn_on(&instance.led[1]);
                ledflag = 1U;
            }
            proc = PROC_SLAVE_END;
            break;

        case PROC_SLAVE_END:
            ts_i2c_set_slave_mode();
            proc = PROC_IDLE;
            break;

        default:
            proc = PROC_IDLE;
            led_turn_on(&instance.led[2]);
            break;
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    switch (timer)
    {
    case BSP_TIMER_1MS:
        timer_irq_handler(&instance.timer);
        break;
    default:
        /* no processing */
        break;
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c(void)
{
   ts_i2cst_irq_handler();
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C TBE IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c_tbe(void)
{
    ts_i2ctberbf_irq_handler();
}

/*--------------------------------------------------*/
/** 
  * @brief  I2C RBF IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c_rbf(void)
{
    ts_i2ctberbf_irq_handler();
}
#ifdef __DEBUG__
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif

/**
 *  @}
 */ /* End of group I2C_SlaveTrans_Exported_functions */

/**
 *  @} 
 */ /* End of group I2C_SlaveTrans */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
