/** 
 *******************************************************************************
 * @file    tspi.h
 * @brief   This file is wrapper file for TSPI driver.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __TSPI_H
#define __TSPI_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "driver_com.h"
#include "tspi_config.h"

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Exported_Typedef TSPI Exported Typedef
 *  @{
 */
/*----------------------------------*/
/** 
 * @struct tspi_receive8_t
 * @brief  Receive event information structure definenition.
 * @brief  When data length definenition is "8bit"( @ref TSPI_DataLength ), use this.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t *p_data;     /*!< The buffer to receive data.   */
    uint32_t num;        /*!< The number of receive data.   */
} tspi_receive8_t;

/*----------------------------------*/
/** 
 * @struct tspi_receive16_t
 * @brief  Receive event information structure definenition.
 * @brief  When data length definenition is "9 - 16 bit"( @ref TSPI_DataLength ), use this.
*/
/*----------------------------------*/
typedef struct
{
    uint16_t *p_data;    /*!< The buffer to receive data.   */
    uint32_t num;        /*!< The number of receive data.   */
} tspi_receive16_t;

/** 
 * @struct tspi_receive32_t
 * @brief  Receive event information structure definenition.
 * @brief  When data length definenition is "17 - 32 bit"( @ref TSPI_DataLength ), use this.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t *p_data;    /*!< The buffer to receive data.   */
    uint32_t num;        /*!< The number of receive data.   */
} tspi_receive32_t;

/*----------------------------------*/
/** 
 * @struct tspi_receive_t
 * @brief  Receive event information structure definenition.
*/
/*----------------------------------*/
typedef union
{
    tspi_receive8_t  rx8;   /*!< @ref tspi_receive8_t  */
    tspi_receive16_t rx16;  /*!< @ref tspi_receive16_t */
    tspi_receive32_t rx32;  /*!< @ref tspi_receive16_t */
} tspi_receive_t;

/*----------------------------------*/
/** 
 * @struct tspi_transmit8_t
 * @brief  Transmit data information structure definenition.
 * @brief  When data length definenition is "8bit"( @ref TSPI_DataLength ), use this.
*/
/*----------------------------------*/
typedef struct
{
    uint8_t *p_data;     /*!< The buffer to transmit data.   */
    uint32_t num;        /*!< The number of transmit data.   */
} tspi_transmit8_t;

/*----------------------------------*/
/** 
 * @struct tspi_transmit16_t
 * @brief  Transmit data information structure definenition.
 * @brief  When data length definenition is "9 - 16 bit"( @ref TSPI_DataLength ), use this.
*/
/*----------------------------------*/
typedef struct
{
    uint16_t *p_data;    /*!< The buffer to transmit data.   */
    uint32_t num;        /*!< The number of transmit data.   */
} tspi_transmit16_t;
/*----------------------------------*/
/** 
 * @struct tspi_transmit32_t
 * @brief  Transmit data information structure definenition.
 * @brief  When data length definenition is "17 - 32 bit"( @ref TSPI_DataLength ), use this.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t *p_data;    /*!< The buffer to transmit data.   */
    uint32_t num;        /*!< The number of transmit data.   */
} tspi_transmit32_t;

/*----------------------------------*/
/** 
 * @struct tspi_transmit_t
 * @brief  Transmit data information structure definenition.
*/
/*----------------------------------*/
typedef union
{
    tspi_transmit8_t  tx8;   /*!< @ref tspi_transmit8_t  */
    tspi_transmit16_t tx16;  /*!< @ref tspi_transmit16_t */
    tspi_transmit32_t tx32;  /*!< @ref tspi_transmit16_t */
} tspi_transmit_t;

/*----------------------------------*/
/** 
 * @brief  TSPI handle structure definenition.
*/
/*----------------------------------*/
typedef struct tspi_handle
{
    TSB_TSPI_TypeDef       *p_instance;     /*!< Registers base address.           */
    tspi_initial_setting_t init;            /*!< Initial setting.                  */
    uint32_t errcode;                       /*!< ErrorCode                         */
    /*------------------------------------------*/
    /*!
        @brief Transmit Informatin.
    */
    /*------------------------------------------*/
    struct
    {
        uint32_t        rp;                                         /*!< Num of transmited data.            */
        uint32_t        fifo_size;                                  /*!< Transmit FIFO size                 */
        tspi_transmit_t info;                                       /*!< Transmit Data Information.         */
        uint8_t tx_allign;                                          /*!< Transmit Data length Information.  */
        void (*handler)(uint32_t id, TXZ_Result result);            /*!< Transmit Event handler.            */
    } transmit;
    /*------------------------------------------*/
    /*!
        @brief Receive Informatin.
    */
    /*------------------------------------------*/
    struct
    {
        uint32_t        rp;                                                            /*!< Num of received data.            */
        uint32_t        fifo_size;                                                     /*!< Receive FIFO size                 */
        tspi_receive_t info;                                                           /*!< Receive Data Information.         */
        uint8_t rx_allign;                                                             /*!< Receive Data length Information.  */
        void (*handler)(uint32_t id, TXZ_Result result, tspi_receive_t *p_info);       /*!< Receive Event handler.            */
    } receive;
} tspi_t;
/**
 *  @}
 */ /* End of group TSPI_Exported_Typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TSPI_Exported_functions TSPI Exported Functions
 *  @{
 */
TXZ_Result tspi_init(tspi_t *p_obj);
TXZ_Result tspi_deinit(tspi_t *p_obj);
TXZ_Result tspi_format(tspi_t *p_obj);
TXZ_Result tspi_start(tspi_t *p_obj);
TXZ_Result tspi_stop(tspi_t *p_obj);
TXZ_Result tspi_write(tspi_t *p_obj, tspi_transmit_t *p_info, uint32_t timeout);
TXZ_Result tspi_read(tspi_t *p_obj, tspi_receive_t *p_info, uint32_t timeout);
TXZ_Result tspi_transfer(tspi_t *p_obj, tspi_transmit_t *p_info);
TXZ_Result tspi_receive(tspi_t *p_obj, tspi_receive_t *p_info);
void tspi_irq_handler_transmit(tspi_t *p_obj);
void tspi_irq_handler_receive(tspi_t *p_obj);
void tspi_error_irq_handler(tspi_t *p_obj);
TXZ_Result tspi_get_status(tspi_t *p_obj, uint32_t *p_status);
TXZ_Result tspi_get_error(tspi_t *p_obj, uint32_t *p_error);
TXZ_Result tspi_error_clear(tspi_t *p_obj);
TXZ_Result tspi_discard_transmit(tspi_t *p_obj);
TXZ_Result tspi_discard_receive(tspi_t *p_obj);

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __TSPI_H */
