/**
 *******************************************************************************
 * @file    i2c.c
 * @brief   This file provides API functions for I2C Class.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "i2c.h"

/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup I2C
 *  @brief      I2C Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_macro */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_define 
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_define 
 *  @{
 */

/** 
 *  @}
 */ /* End of group I2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_typedef
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group I2C_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_variables
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group I2C_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Const Table                                                                 */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_const
 *  @{
 */
/*----------------------------------*/
/** 
 * @brief   SCK Divider value table.
 * @details SCK  = b000 - b111.
 * @note    NFSEL=0 (Digital Setting) Divider value.
*/
/*----------------------------------*/
static const uint32_t I2C_SCK_DIVIDER_TBL[8] = { 20,24,32,48,80,144,272,528 };
static const uint32_t I2C_SCK_LOW_MUL_TBL[8] = { 12, 14, 18, 26, 42, 74, 138, 266 };

/**
 *  @}
 */ /* End of group I2C_Private_const */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Private_functions
 *  @{
 */
 
#ifdef __DEBUG__
__STATIC_INLINE int32_t check_param_sck(uint32_t param);
__STATIC_INLINE int32_t check_param_prsck(uint32_t param);
__STATIC_INLINE int32_t check_param_interrupt_selpincd(uint32_t param);
__STATIC_INLINE int32_t check_param_interrupt_dmari2ctx(uint32_t param);
__STATIC_INLINE int32_t check_param_interrupt_dmari2crx(uint32_t param);
__STATIC_INLINE int32_t check_param_interrupt_intnack(uint32_t param);
__STATIC_INLINE int32_t check_param_interrupt_inti2cbf(uint32_t param);
__STATIC_INLINE int32_t check_param_interrupt_inti2cal(uint32_t param);
__STATIC_INLINE int32_t check_param_interrupt_inti2c(uint32_t param);
#if (I2CS_EQUIPPED == AVAILABLE)
__STATIC_INLINE int32_t check_param_wup_sgcdi(uint8_t param);
__STATIC_INLINE int32_t check_param_wup_ack(uint8_t param);
__STATIC_INLINE int32_t check_param_wup_reset(uint8_t param);
__STATIC_INLINE int32_t check_param_wup_intend(uint8_t param);
#endif /* (I2CS_EQUIPPED == AVAILABLE) */
#endif /* #ifdef __DEBUG__ */

#ifdef __DEBUG__
/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_sck(uint32_t param);
  * @brief  Check the SCK's parameter
  * @param[in] param :SCK's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_sck(uint32_t param)
{
     int32_t result = I2C_PARAM_NG;

     if (param < I2C_SCK_MAX)
     {
        result = I2C_PARAM_OK;
     }

     return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_prsck(uint32_t param)
  * @brief  Check the PRSCK's parameter
  * @param[in] param :PRSCK's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_prsck(uint32_t param)
{
     int32_t result = I2C_PARAM_NG;

     if (param < I2C_PRSCK_MAX)
     {
        result = I2C_PARAM_OK;
     }

     return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_interrupt_selpincd(uint32_t param)
  * @brief  Check the SELPINCD's parameter
  * @param[in] param :SELPINCD's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interrupt_selpincd(uint32_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxIE_SELPINCD_ENABLE:
    case I2CxIE_SELPINCD_DISABLE:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_interrupt_dmari2ctx(uint32_t param)
  * @brief  Check the DMARI2CTX's parameter
  * @param[in] param :DMARI2CTX's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interrupt_dmari2ctx(uint32_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxIE_DMARI2CTX_ENABLE:
    case I2CxIE_DMARI2CTX_DISABLE:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_interrupt_dmari2crx(uint32_t param)
  * @brief  Check the DMARI2CRX's parameter
  * @param[in] param :DMARI2CRX's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interrupt_dmari2crx(uint32_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxIE_DMARI2CRX_ENABLE:
    case I2CxIE_DMARI2CRX_DISABLE:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_interrupt_intnack(uint32_t param)
  * @brief  Check the INTNACK's parameter
  * @param[in] param :INTNACK's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interrupt_intnack(uint32_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxIE_INTNACK_ENABLE:
    case I2CxIE_INTNACK_DISABLE:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_interrupt_inti2cbf(uint32_t param)
  * @brief  Check the INTI2CBF's parameter
  * @param[in] param :INTI2CBF's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interrupt_inti2cbf(uint32_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxIE_INTI2CBF_ENABLE:
    case I2CxIE_INTI2CBF_DISABLE:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_interrupt_inti2cal(uint32_t param)
  * @brief  Check the INTI2CAL's parameter
  * @param[in] param :INTI2CAL's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interrupt_inti2cal(uint32_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxIE_INTI2CAL_ENABLE:
    case I2CxIE_INTI2CAL_DISABLE:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_interrupt_inti2c(uint32_t param)
  * @brief  Check the INTI2C's parameter
  * @param[in] param :INTI2C's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_interrupt_inti2c(uint32_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxIE_INTI2C_ENABLE:
    case I2CxIE_INTI2C_DISABLE:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

#if (I2CS_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_wup_sgcdi(uint8_t param)
  * @brief  Check the SGCDI's parameter
  * @param[in] param :SGCDI's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_wup_sgcdi(uint8_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxWUPCR1_SGCDI_OFF:
    case I2CxWUPCR1_SGCDI_ON:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_wup_ack(uint8_t param)
  * @brief  Check the ACK's parameter
  * @param[in] param :ACK's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_wup_ack(uint8_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxWUPCR1_ACK_ACK:
    case I2CxWUPCR1_ACK_NACK:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_wup_reset(uint8_t param)
  * @brief  Check the I2RES's parameter
  * @param[in] param :I2RES's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_wup_reset(uint8_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxWUPCR1_I2RES_RESET:
    case I2CxWUPCR1_I2RES_RELEASE:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_wup_intend(uint8_t param)
  * @brief  Check the INTEND's parameter
  * @param[in] param :INTEND's parameter
  * @retval    I2C_PARAM_OK :Success.
  * @retval    I2C_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE int32_t check_param_wup_intend(uint8_t param)
{
    int32_t result = I2C_PARAM_NG;

    switch (param) 
    {
    case I2CxWUPCR1_INTEND_RELEASE:
    case I2CxWUPCR1_INTEND_HOLD:
        result = I2C_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }
    return (result);
}
#endif /* (I2CS_EQUIPPED == AVAILABLE) */

#endif /* #ifdef __DEBUG__ */

/**
 *  @}
 */ /* End of group I2C_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup I2C_Exported_functions
 *  @{
 */

/*--------------------------------------------------*/
/**
  * @brief     Initializing I2C Regester
  * @param     p_obj       :I2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void I2C_init(I2C_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_sck(p_obj->init.clock.sck));
    assert_param(check_param_prsck(p_obj->init.clock.prsck));
#endif /* #ifdef __DEBUG__ */
    p_obj->p_instance->CR2 = I2CxCR2_I2CM_ENABLE;
    p_obj->p_instance->OP  = I2CxOP_INIT;
    p_obj->p_instance->CR1 = (I2CxCR1_ACK | I2CxCR1_NOACK | p_obj->init.clock.sck);
    p_obj->p_instance->AR  = I2CxAR_INIT;
    p_obj->p_instance->AR2 = I2CxAR2_INIT;
    p_obj->p_instance->CR2 = I2CxCR2_INIT;
    p_obj->p_instance->PRS = (I2CxPRS_PRCK & p_obj->init.clock.prsck);
    p_obj->p_instance->IE  = I2CxIE_CLEAR;
}

/*--------------------------------------------------*/
/**
  * @brief     Generate start condition
  * @param     p_obj       :I2C object.
  * @param     data        :Slave address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void I2C_start_condition(I2C_t *p_obj, uint32_t data)
{
    __IO uint32_t opreg;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_sck(p_obj->init.clock.sck));
#endif /* #ifdef __DEBUG__ */

    opreg = p_obj->p_instance->OP;
    opreg &= ~(I2CxOP_RSTA | I2CxOP_SREN);
    if(I2C_master(p_obj)){
        if ((p_obj->p_instance->SR & I2CxSR_BB))
        {
            opreg |= I2CxOP_SREN;
        }
    }
    p_obj->p_instance->CR1 = (I2CxCR1_ACK | I2CxCR1_NOACK | p_obj->init.clock.sck);
    p_obj->p_instance->OP  = opreg;
    p_obj->p_instance->DBR = (data & I2CxDBR_DB_MASK);
    p_obj->p_instance->CR2 = I2CxCR2_START_CONDITION;
}

/*--------------------------------------------------*/
/**
  * @brief     Return the I2c clock setting
  * @param     frequency   :Maximum frequency.
  * @param     fsys        :SystemCoreClock.
  * @param     p_setting   :Clock data pointer.
  * @retval    Non-zero    :Scl frequency.
  * @retval    0           :Error.
  * @note      -
  */
/*--------------------------------------------------*/
uint32_t I2C_get_clock_setting(uint32_t frequency, uint32_t fsys, I2C_clock_setting_t *p_setting)
{
    uint32_t result = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_setting));
#endif /* #ifdef __DEBUG__ */

    if (frequency <= 1000000)
    {
        uint64_t sck, tmp_sck;
        uint64_t prsck, tmp_prsck;
        uint64_t fscl, tmp_fscl;
        uint64_t fx;
        uint64_t max_fx, min_fx;
        uint64_t low_width,low_width_min;

        sck       = 0;
        tmp_sck   = 0;
        prsck     = 1;
        tmp_prsck = 1;
        fscl      = 0;
        tmp_fscl  = 0;
        
        /* STD */
        if(frequency <= 100000)
        {
            max_fx = 20000000U;     /* Tpresck: 50.0ns 1/ 50  = 0.0200000000 */
            min_fx =  6666666U;     /* Tpresck:150.0ns 1/150  = 0.0066666667 */
            low_width_min = 5000;   /* 4.7(us)(low width min) + 0.3(us)(falling time) */
        }
        /* FM */
        else if(frequency <= 400000)
        {
            max_fx = 20000000U;     /* Tpresck: 50.0ns 1/ 50  = 0.0200000000 */
            min_fx =  6666666U;     /* Tpresck:150.0ns 1/150  = 0.0066666667 */
            low_width_min = 1600;   /* 1.3(us)(low width min) + 0.3(us)(falling time) */
        }
        /* FMP */
        else
        {
            max_fx = 50000000U;     /* Tpresck:20.0ns 1/20    = 0.0500000000 */
            min_fx = 15384615U;     /* Tpresck:65.0ns 1/65    = 0.0153846154 */
            low_width_min = 620;    /* 0.5(us)(low width min) + 0.12(us)(falling time) */
        }
        for (prsck = 1; prsck <= 32; prsck++)
        {
            fx = ((uint64_t)fsys / prsck);

            if ((fx < max_fx) && (fx >= min_fx))
            {
                for (sck = 0; sck <= 7; sck++)
                {
                    low_width = (uint64_t)(1000000000 * prsck * I2C_SCK_LOW_MUL_TBL[sck]) / fsys;
                    if(low_width < low_width_min)
                    {
                        continue;
                    }
                    fscl = (fx / (uint64_t)I2C_SCK_DIVIDER_TBL[sck]);

                    if ((fscl <= frequency) && (fscl > tmp_fscl))
                    {
                        tmp_fscl = fscl;
                        tmp_sck = sck;
                        tmp_prsck = (prsck < (uint64_t)32)? prsck: (uint64_t)0;
                    }
                }
            }
        }
        result = (uint32_t)tmp_fscl;
        p_setting->sck = (uint32_t)tmp_sck;
        p_setting->prsck = (uint32_t)(tmp_prsck < 32)? (uint32_t)tmp_prsck: (uint32_t)0;
    }
    else
    {
        result = 0;
    }
    return (result);
}

/*--------------------------------------------------*/
/**
  * @brief     Slave mode setting.
  * @param     p_obj       :I2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void I2C_slave_init(I2C_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_sck(p_obj->init.clock.sck));
    assert_param(check_param_prsck(p_obj->init.clock.prsck));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->OP  = I2CxOP_SLAVE_INIT;
    p_obj->p_instance->CR1 = (I2CxCR1_ACK | p_obj->init.clock.sck);
    p_obj->p_instance->CR2 = (I2CxCR2_INIT | I2CxCR2_PIN_CLEAR);
    p_obj->p_instance->CR2 = I2CxCR2_INIT;
    p_obj->p_instance->PRS = (I2CxPRS_PRCK & p_obj->init.clock.prsck);
}

/*--------------------------------------------------*/
/**
  * @brief     interrupt setting.
  * @param     p_obj       :I2C object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void I2C_interrupt_setting(I2C_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_interrupt_selpincd(p_obj->init.interrupt.selpincd));
    assert_param(check_param_interrupt_dmari2ctx(p_obj->init.interrupt.dmari2ctx));
    assert_param(check_param_interrupt_dmari2crx(p_obj->init.interrupt.dmari2crx));
    assert_param(check_param_interrupt_intnack(p_obj->init.interrupt.intnack));
    assert_param(check_param_interrupt_inti2cbf(p_obj->init.interrupt.inti2cbf));
    assert_param(check_param_interrupt_inti2cal(p_obj->init.interrupt.inti2cal));
    assert_param(check_param_interrupt_inti2c(p_obj->init.interrupt.inti2c));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->IE = (p_obj->init.interrupt.selpincd | p_obj->init.interrupt.dmari2ctx | p_obj->init.interrupt.dmari2crx |
                             p_obj->init.interrupt.intnack | p_obj->init.interrupt.inti2cbf | p_obj->init.interrupt.inti2cal |
                             p_obj->init.interrupt.inti2c );
}


#if (I2CS_EQUIPPED == AVAILABLE)
/*--------------------------------------------------*/
/**
  * @brief     I2C Wakeup Control setting.
  * @param     p_obj       :I2CS object.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void I2CS_init(I2CS_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_wup_sgcdi(p_obj->init.wup.sgcdi));
    assert_param(check_param_wup_ack(p_obj->init.wup.ack));
    assert_param(check_param_wup_reset(p_obj->init.wup.i2res));
    assert_param(check_param_wup_intend(p_obj->init.wup.intend));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->WUPCR1 = (p_obj->init.wup.sgcdi | p_obj->init.wup.ack | p_obj->init.wup.i2res | p_obj->init.wup.intend);
}

/*--------------------------------------------------*/
/**
  * @brief     Get I2C Wakeup Control status.
  * @param     p_obj       :I2CS object.
  * @param     p_status    :I2CS status. Destination address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void I2CS_get_status(I2CS_t *p_obj, uint8_t *p_status)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(IS_POINTER_NOT_NULL(p_status));
#endif /* #ifdef __DEBUG__ */

    *p_status = p_obj->p_instance->WUPSL;
}

/*--------------------------------------------------*/
/**
  * @brief     Primary Slave Address setting.
  * @param     p_obj       :I2CS object.
  * @param     addr        :Primary Slave Address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void I2CS_Primary_slave_adr_set(I2CS_t *p_obj, uint8_t adr)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->WUPCR2 = (0xFE & adr);
}

/*--------------------------------------------------*/
/**
  * @brief     Secondary Slave Address setting.
  * @param     p_obj       :I2CS object.
  * @param     addr        :Secondary Slave Address.
  * @retval    -
  * @note      -
  */
/*--------------------------------------------------*/
void I2CS_Secondary_slave_adr_set(I2CS_t *p_obj, uint8_t adr)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->WUPCR3 = (0xFE & adr);
    p_obj->p_instance->WUPCR3 |= 0x01;        /* WUPSA2EN: Secondary Slave Address Use Setting */
}
#endif /* (I2CS_EQUIPPED == AVAILABLE) */
/**
 *  @}
 */ /* End of group I2C_Exported_functions */

/**
 *  @} 
 */ /* End of group I2C */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */

