/**
 *******************************************************************************
 * @file    trm_b.c
 * @brief   This file provides API functions for TRM.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "trm.h"

#if defined(__TRM_B_H)
/**
 *  @addtogroup Periph_Driver Peripheral Driver
 *  @{
 */

/** 
 *  @addtogroup TRM
 *  @brief      TRM Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRM_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRM_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_define
 *  @{
 */

#define TRMOSCPRO_ENABLE           (0x000000C1UL)      /* Writing register control enable. */
#define TRMOSCPRO_DISABLE          (0x00000000UL)      /* Writing register control disable. */
#define TRMOSCEN_MASK              (0x00000001UL)      /* Trimming Control enable. */
#define TRIM_MASK                  (0x000001FEUL)      /* Trimming value mask. */

/** 
 *  @}
 */ /* End of group TRM_Private_define */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Private_fuctions
 *  @{
 */
#ifdef __DEBUG__
    __STATIC_INLINE  int32_t check_param_trim_control(uint32_t param);
    __STATIC_INLINE  int32_t check_param_trim_value(uint32_t param);

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_trim_control(uint32_t param)
  * @brief  Check the Value to TRIMEN's parameter.
  * @param[in] param :Value to TRIMEN's parameter
  * @retval    TRM_PARAM_OK :Success.
  * @retval    TRM_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE  int32_t check_param_trim_control(uint32_t param)
{
    int32_t result = TRM_PARAM_NG;

    switch (param) 
    {
    case TRMOSC_CONTROL_ENABLE:
    case TRMOSC_CONTROL_DISABLE:
        result = TRM_PARAM_OK;
        break;
    default:
        /* no process */
        break;
    }

    return (result);
}

/*--------------------------------------------------*/
/*!
  * @fn     static int32_t check_param_trim_value(uint32_t param)
  * @brief  Check the Value to TRIMSETC's parameter.
  * @param[in] param :Value to TRIMSETC's parameter
  * @retval    TRM_PARAM_OK :Success.
  * @retval    TRM_PARAM_NG :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
__STATIC_INLINE  int32_t check_param_trim_value(uint32_t param)
{
    int32_t result = TRM_PARAM_NG;

    if (param <= TRM_VALUE_MAX)
    {
        result = TRM_PARAM_OK;
    }

    return (result);
}
#endif /* #ifdef __DEBUG__ */
/**
 *  @}
 */ /* End of group TRM_Private_fuctions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TRM_Exported_functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Set TRM register
  * @param[in] p_obj  :TRM object.
  * @param  TRMOSC Control Structure
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void SetTrmOsc(trm_t *p_obj, TRM_ControlTypeDef *ControlStruct)
{
    uint32_t tmp = 0;

    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TRM_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
    assert_param(check_param_trim_control(ControlStruct->TrimmingControl));
    assert_param(check_param_trim_value(ControlStruct->TrimmingValue));
#endif /* #ifdef __DEBUG__ */

    p_obj->p_instance->OSCPRO = TRMOSCPRO_ENABLE;
    /* OSCEN disable change to enable set OSCET1,OSCET2 */
    if((p_obj->p_instance->OSCEN == TRMOSC_CONTROL_DISABLE)
    &&(ControlStruct->TrimmingControl == TRMOSC_CONTROL_ENABLE))
    {
        p_obj->p_instance->OSCSET0 = p_obj->p_instance->OSCINIT0;
        p_obj->p_instance->OSCSET1 = p_obj->p_instance->OSCINIT1;
        p_obj->p_instance->OSCSET2 = p_obj->p_instance->OSCINIT2;
    }
    p_obj->p_instance->OSCEN  = ControlStruct->TrimmingControl;
    tmp = (ControlStruct->TrimmingValue << 1) & TRIM_MASK;
    p_obj->p_instance->OSCSET0 = tmp;
    p_obj->p_instance->OSCPRO = TRMOSCPRO_DISABLE;
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Trimming Control Value
  * @param[in] p_obj  :TRM object.
  * @retval Trimming Control value
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetTrimmingControlValue(trm_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TRM_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    return (p_obj->p_instance->OSCEN & TRMOSCEN_MASK);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Initial Trimming Value
  * @param[in] p_obj  :TRM object.
  * @retval Initial Trimming value
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetInitTrimValue(trm_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TRM_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    return ((p_obj->p_instance->OSCINIT0 & TRIM_MASK) >> 1);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Seting Trimming value 
  * @param[in] p_obj  :TRM object.
  * @retval setting Trimming value 
  * @note   -
  */
/*--------------------------------------------------*/
uint32_t GetSetTrimValue(trm_t *p_obj)
{
    /*------------------------------*/
    /*  Parameter Check             */
    /*------------------------------*/
#ifdef __DEBUG__
    /* Check the TRM_NULL of address. */
    assert_param(IS_POINTER_NOT_NULL(p_obj));
    assert_param(IS_POINTER_NOT_NULL(p_obj->p_instance));
#endif /* #ifdef __DEBUG__ */

    return ((p_obj->p_instance->OSCSET0 & TRIM_MASK) >> 1);
}

/**
 *  @}
 */ /* End of group TRM_Exported_functions */

/**
 *  @}
 */ /* End of group TRM */

/**
 *  @} 
 */ /* End of group Periph_Driver */

#endif /* defined(__TRM_B_H)  */
#ifdef __cplusplus
}
#endif /* __cplusplus */


