/** 
 *******************************************************************************
 * @file    tspi_dma_e.h
 * @brief   This file provides all the functions prototypes for TSPI driver.
 * @brief   If DMA transfer is used, it is necessary.
 * @version V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2022
 *******************************************************************************
 */
/*------------------------------------------------------------------------------*/
/*  Define to prevent recursive inclusion                                       */
/*------------------------------------------------------------------------------*/
#ifndef __TSPI_DMA_E_H
#define __TSPI_DMA_E_H

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "driver_com.h"
#include "tspi.h"
#include "dmac_ex.h"

/**
 *  @addtogroup Periph_Driver
 *  @{
 */

/** 
 *  @addtogroup TSPI
 *  @brief      TSPI Driver.
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Exported_define
 *  @{
 */

/** 
 *  @defgroup TSPI_DMA_TransferType  Transfer Type
 *  @brief    DMA transfer type.
 *  @{
 */
#define TSPI_DMA_TYPE_UNUSE             ((uint32_t)0)    /*!< Unuse DMA.                */
#define TSPI_DMA_TYPE_MEM_TO_MEM        ((uint32_t)1)    /*!< Memory to Memory.         */
#define TSPI_DMA_TYPE_MEM_TO_PERI       ((uint32_t)2)    /*!< Memory to Peripheral.     */
#define TSPI_DMA_TYPE_PERI_TO_MEM       ((uint32_t)3)    /*!< Peripheral to Memory.     */
#define TSPI_DMA_TYPE_PERI_TO_PERI      ((uint32_t)4)    /*!< Peripheral to Peripheral. */
/**
 *  @}
 */ /* End of group TSPI_DMA_TransferType */

/**
 *  @}
 */ /* End of group TSPI_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Exported_define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group TSPI_Exported_define */


/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Exported_Typedef
 *  @{
 */

/*----------------------------------*/
/** 
 * @brief  DMA setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    uint32_t type;      /*!< DMA transfer type. 
                            : Use @ref TSPI_DMA_TransferType          */
    uint32_t ch;        /*!< DMA tspi channel.  
                            : If DMA isn't used, it isn't necessary.  */
} tspi_dma_setting_t;

/*----------------------------------*/
/** 
 * @brief  TSPI DMA initial setting structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    dma_t              *p_obj;  /*!< DMA object address.  */
    tspi_dma_setting_t tx;      /*!< Transmit setting.    */
    tspi_dma_setting_t rx;      /*!< Receive setting.     */
} tspi_dma_initial_setting_t;

/*----------------------------------*/
/** 
 * @brief  TSPI DMA handle structure definenition.
*/
/*----------------------------------*/
typedef struct
{
    tspi_t super;                       /*!< Uart super class. 
                                            : Use @ref tspi_t                       */
    tspi_dma_initial_setting_t init;    /*!< Initial setting.
                                            : Use @ref tspi_dma_initial_setting_t   */
    uint32_t work[8];                   /*!< Work area.                             */
} tspi_dma_t;

/**
 *  @}
 */ /* End of group TSPI_Exported_Typedef */


/*------------------------------------------------------------------------------*/
/*  Functions                                                                   */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup TSPI_Exported_functions
 *  @{
 */
TXZ_Result tspi_dma_init(tspi_dma_t *p_obj);
TXZ_Result tspi_dma_deinit(tspi_dma_t *p_obj);
TXZ_Result tspi_dma_discard_transmit(tspi_dma_t *p_obj);
TXZ_Result tspi_dma_discard_receive(tspi_dma_t *p_obj);
TXZ_Result tspi_dma_transmitIt(tspi_dma_t *p_obj, tspi_transmit_t *p_info);
TXZ_Result tspi_dma_receiveIt(tspi_dma_t *p_obj, tspi_receive_t *p_info);
/**
 *  @}
 */ /* End of group TSPI_Exported_functions */

/**
 *  @}
 */ /* End of group TSPI */

/**
 *  @}
 */ /* End of group Periph_Driver */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __TSPI_DMA_E_H */


